#include <SPI.h>
#include <avr/pgmspace.h>

#include "ff.h"
#include "diskio.h"

/*
  Blink
  Turns on an LED on for one second, then off for one second, repeatedly.
 
  This example code is in the public domain.
 */
 
// Pin 13 has an LED connected on most Arduino boards.
// give it a name:
static const int pwm = 9;

static unsigned int spid = 1000;
static unsigned int space = 1000;
static byte h, repeat = 0;

static FIL file;
static FATFS FatFs;
static byte ofs;
static byte screen[64];

static void echo(byte b) {
  PORTC = (PORTC & 0xC0) | (b & 0x3F);
  PORTD = (PORTD & 0xF3) | ((b >> 4) & 0x0C);
}

static void fatal(byte err) {
  for (;;) {
    echo(err);
    delay(100);
    echo(0);
    delay(100);
  }
}

static boolean readline(void) {
  char *line = (char*)screen;
  char *p;
  char c;
  UINT nr;
  
  do {
    p = f_gets(line, sizeof(screen), &file);
    if (!p)
      return false;
      
    p = strchr(line, '\n');
    if (!p) {
      do {
        f_read(&file, &c, 1, &nr);
      } while (c != '\n' && nr);
    } else {
      *p = 0;
    }
  } while (*line == '#');
  
  return true;
}

// the setup routine runs once when you press reset:
void setup() {
  UINT nrd;
  char *line = (char*)screen;
  
  SPI.begin();
  
  // initialize the digital pin as an output.
  pinMode(pwm, OUTPUT);
  digitalWrite(pwm, LOW);
  //analogWrite(pwm, 240);
  
  DDRC = 0x3F;
  DDRD |= 0x0C;
  
  if (disk_initialize(0))
    fatal(1);
  
  FRESULT r;
  r = f_mount(&FatFs, "0:", 1);
  if (r) fatal(2);
  
  // load config
  r = f_open(&file, "0:config.txt", FA_READ | FA_OPEN_EXISTING);
  if (!r) {
    while (readline()) {
      char *p;
      
      p = strchr(line, ':');
      if (!p) continue;
      for (*p++ = 0; *p == ' '; ++p);
      
      if (!strcmp_P(line, PSTR("delay"))) {
        spid = atoi(p);
      } else if (!strcmp_P(line, PSTR("space"))) {
        space = atoi(p);
      } else if (!strcmp_P(line, PSTR("repeat"))) {
        repeat = atoi(p);
      } else if (!strcmp_P(line, PSTR("light"))) {
        byte intsy = 255 - (unsigned int)atoi(p) * 51 / 20;
        if (intsy) {
          analogWrite(pwm, intsy);
        }
      }
    }
  }
  
  // load picture
  r = f_open(&file, "0:poi.pbm", FA_READ | FA_OPEN_EXISTING);
  if (r) fatal(3);
  
  readline();
  if (strcmp_P(line, PSTR("P4"))) fatal(4);
  
  {
    char *p;
    int i;
    
    readline();
    i = atoi(line);
    if (i != 8) fatal(5);
    p = strchr(line, ' ');
    if (!p) fatal(5);
    i = atoi(p + 1);
    if ((unsigned int)i > sizeof(screen)) fatal(5);
    h = i;
  }
  
  f_read(&file, screen, h, &nrd);
  if (nrd != h) fatal(6);
}


// the loop routine runs over and over again forever:
void loop() {
  byte b;
  
  if (++ofs >= h) {
    ofs = 0;
    delay(repeat);
  }
  b = screen[ofs];

  echo(b);
  delayMicroseconds(spid);
  echo(0);
  delayMicroseconds(space);
}
